@@ -1252,7 +1252,8 @@ static void HandleDefinition() {
     if (Function *LF = F->Codegen()) {
 #ifndef MINIMAL_STDERR_OUTPUT
       fprintf(stderr, "Read function definition:");
-      LF->dump();
+      LF->print(errs());
+      fprintf(stderr, "\n");
 #endif
     }
   } else {
@@ -1266,7 +1267,8 @@ static void HandleExtern() {
     if (Function *F = P->Codegen()) {
 #ifndef MINIMAL_STDERR_OUTPUT
       fprintf(stderr, "Read extern: ");
-      F->dump();
+      F->print(errs());
+      fprintf(stderr, "\n");
 #endif
     }
   } else {
@@ -1370,7 +1372,7 @@ int main() {
 
 #ifndef MINIMAL_STDERR_OUTPUT
   // Print out all of the generated code.
-  TheHelper->dump();
+  TheHelper->print(errs());
 #endif
 
   return 0;
