@@ -14,9 +14,10 @@
 using namespace llvm;
 
 llvm::detail::ErrorHolder llvm::detail::TakeError(llvm::Error Err) {
-  bool Succeeded = !static_cast<bool>(Err);
-  std::string Message;
-  if (!Succeeded)
-    Message = toString(std::move(Err));
-  return {Succeeded, Message};
+  std::vector<std::shared_ptr<ErrorInfoBase>> Infos;
+  handleAllErrors(std::move(Err),
+                  [&Infos](std::unique_ptr<ErrorInfoBase> Info) {
+                    Infos.emplace_back(std::move(Info));
+                  });
+  return {std::move(Infos)};
 }
