@@ -24,7 +24,7 @@
 #include "llvm/MC/MCSectionMachO.h"
 #include "llvm/MC/MCStreamer.h"
 #include "llvm/MC/MCSubtargetInfo.h"
-#include "llvm/MC/MCTargetOptionsCommandFlags.def"
+#include "llvm/MC/MCTargetOptionsCommandFlags.inc"
 #include "llvm/Support/MemoryBuffer.h"
 #include "llvm/Support/CommandLine.h"
 #include "llvm/Support/FileUtilities.h"
@@ -190,8 +190,8 @@ int AssembleOneInput(const uint8_t *Data, size_t Size) {
   const char *ProgName = "llvm-mc-fuzzer";
   std::unique_ptr<MCSubtargetInfo> STI(
       TheTarget->createMCSubtargetInfo(TripleName, MCPU, FeaturesStr));
-  MCCodeEmitter *CE = nullptr;
-  MCAsmBackend *MAB = nullptr;
+  std::unique_ptr<MCCodeEmitter> CE = nullptr;
+  std::unique_ptr<MCAsmBackend> MAB = nullptr;
 
   MCTargetOptions MCOptions = InitMCTargetOptionsFromFlags();
 
@@ -202,9 +202,9 @@ int AssembleOneInput(const uint8_t *Data, size_t Size) {
   std::unique_ptr<MCStreamer> Str;
 
   if (FileType == OFT_AssemblyFile) {
-    Str.reset(TheTarget->createAsmStreamer(
-        Ctx,  std::move(FOut), AsmVerbose,
-        UseDwarfDirectory, IP, CE, MAB, ShowInst));
+    Str.reset(TheTarget->createAsmStreamer(Ctx, std::move(FOut), AsmVerbose,
+                                           UseDwarfDirectory, IP, std::move(CE),
+                                           std::move(MAB), ShowInst));
   } else {
     assert(FileType == OFT_ObjectFile && "Invalid file type!");
 
@@ -228,8 +228,7 @@ int AssembleOneInput(const uint8_t *Data, size_t Size) {
     }
 
     MCCodeEmitter *CE = TheTarget->createMCCodeEmitter(*MCII, *MRI, Ctx);
-    MCAsmBackend *MAB = TheTarget->createMCAsmBackend(*MRI, TripleName, MCPU,
-                                                      MCOptions);
+    MCAsmBackend *MAB = TheTarget->createMCAsmBackend(*STI, *MRI, MCOptions);
     Str.reset(TheTarget->createMCObjectStreamer(
         TheTriple, Ctx, std::unique_ptr<MCAsmBackend>(MAB), *OS,
         std::unique_ptr<MCCodeEmitter>(CE), *STI, MCOptions.MCRelaxAll,
