@@ -7,19 +7,16 @@
 //
 //===----------------------------------------------------------------------===//
 
-#include "llvm/ExecutionEngine/RuntimeDyld.h"
 #include "llvm/ExecutionEngine/Orc/LazyEmittingLayer.h"
+#include "llvm/ExecutionEngine/RuntimeDyld.h"
 #include "gtest/gtest.h"
 
 namespace {
 
 struct MockBaseLayer {
-  typedef int ModuleSetHandleT;
-  ModuleSetHandleT addModuleSet(
-                  std::list<std::unique_ptr<llvm::Module>>,
-                  std::unique_ptr<llvm::RuntimeDyld::MemoryManager> MemMgr,
-                  std::unique_ptr<llvm::JITSymbolResolver> Resolver) {
-    EXPECT_FALSE(MemMgr);
+  typedef int ModuleHandleT;
+  ModuleHandleT addModule(llvm::orc::VModuleKey,
+                          std::shared_ptr<llvm::Module>) {
     return 42;
   }
 };
@@ -27,7 +24,8 @@ struct MockBaseLayer {
 TEST(LazyEmittingLayerTest, Empty) {
   MockBaseLayer M;
   llvm::orc::LazyEmittingLayer<MockBaseLayer> L(M);
-  L.addModuleSet(std::list<std::unique_ptr<llvm::Module>>(), nullptr, nullptr);
+  cantFail(
+      L.addModule(llvm::orc::VModuleKey(), std::unique_ptr<llvm::Module>()));
 }
 
 }
